//*************************************************************************
//                                                                        *
//                        IEDownload                                      *
//                       For Delphi                                       *
//                                                                        *
//                     Freeware Component                                 *
//                            by                                          *
//                     Per Linds Larsen                                  *
//                   per.lindsoe@larsen.dk                                *
//                                                                        *
//  Developing Team:                                                      *
//  Eran Bodankin -bsalsa(bsalsa@bsalsa.com) (All the new stuf)           *
//  Mathias Walter (mich@matze.tv)  Attachment, callback, and a lot more  *             
//                                                                        *
//                                                                        *
//  Updated versions:                                                     *
//               http://www.bsalsa.com                                    *
//*************************************************************************
{LICENSE:
THIS SOFTWARE IS PROVIDED TO YOU "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESSED OR IMPLIED INCLUDING BUT NOT LIMITED TO THE APPLIED
WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
YOU ASSUME THE ENTIRE RISK AS TO THE ACCURACY AND THE USE OF THE SOFTWARE
AND ALL OTHER RISK ARISING OUT OF THE USE OR PERFORMANCE OF THIS SOFTWARE
AND DOCUMENTATION. [YOUR NAME] DOES NOT WARRANT THAT THE SOFTWARE IS ERROR-FREE
OR WILL OPERATE WITHOUT INTERRUPTION. THE SOFTWARE IS NOT DESIGNED, INTENDED
OR LICENSED FOR USE IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE CONTROLS,
INCLUDING WITHOUT LIMITATION, THE DESIGN, CONSTRUCTION, MAINTENANCE OR
OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS,
AIR TRAFFIC CONTROL, AND LIFE SUPPORT OR WEAPONS SYSTEMS. VSOFT SPECIFICALLY
DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR SUCH PURPOSE.

You may use, change or modify the component under 3 conditions:
1. In your website, add a link to "http://www.bsalsa.com"
2. In your application, add credits to "Embedded Web Browser"
3. Mail me  (bsalsa@bsalsa.com) any code change in the unit
   for the benefit of the other users.
4. Please, consider donation in our web site!   
{*******************************************************************************}

unit IEDownload;

interface

{$I EWB.inc}

uses
   Dialogs, IEConst, ActiveX, Contnrs, ExtCtrls, Windows, WinInet, UrlMon,
   Classes, SysUtils;

type
   TScheme = record
      Name: string;
      Id: Byte;
   end;

   TState = (Requested, Downloading, Canceled, Finished);

const
   EXT_SCHEMES: array[0..5] of TScheme = (
      (Name: 'socks'; Id: INTERNET_SCHEME_SOCKS),
      (Name: 'javascript'; Id: INTERNET_SCHEME_JAVASCRIPT),
      (Name: 'vbscript'; Id: INTERNET_SCHEME_VBSCRIPT),
      (Name: 'res'; Id: INTERNET_SCHEME_RES),
      (Name: 'about'; Id: INTERNET_SCHEME_ABOUT),
      (Name: 'cookie'; Id: INTERNET_SCHEME_COOKIE));

   BINDSTATUS_SESSION_COOKIES_ALLOWED = 48;
   BINDSTATUS_CACHECONTROL = BINDSTATUS_SESSION_COOKIES_ALLOWED + 1;
   BINDSTATUS_CONTENTDISPOSITIONFILENAME = BINDSTATUS_CACHECONTROL + 1;
   BINDSTATUS_MIMETEXTPLAINMISMATCH = BINDSTATUS_CONTENTDISPOSITIONFILENAME + 1;
   BINDSTATUS_PUBLISHERAVAILABLE = BINDSTATUS_MIMETEXTPLAINMISMATCH + 1;
   BINDSTATUS_DISPLAYNAMEAVAILABLE = BINDSTATUS_PUBLISHERAVAILABLE + 1;

type
   TInfo = record
      Sender: TObject;
      Url: PWideChar;
      TimeOut: Integer;
      FileName: string;
      FileExt: String;
      DownloadDir: string;
      PostData: string;
      PutFileName: string;
      AdditionalHeader: TStrings;
      Password: string;
      UserName: string;
      CustomVerb: string;
      ExtraInfo: string;
      Descriptor: string;
      UserAgent: string;
      RangeBegin: Cardinal;
      RangeEnd: Integer;
      InheritHandle: Boolean;
      BindVerbValue: Cardinal;
      BindInfoValue: Cardinal;
      UrlEncodeValue: Cardinal;
      CodePageValue: Cardinal;
      OpenAfterDownload: Boolean;
      FileSize: Cardinal;
      Index: Integer;
   end;

   TProxySettings = class(TPersistent)
   private
      FPort: Integer;
      FServer: string;
      FAutoLoadProxy: Boolean;
   public
      procedure SetProxy(FullUserAgent, ProxyServer: string);
   published
      property AutoLoadProxy: Boolean read FAutoLoadProxy write FAutoLoadProxy default False;
      property Port: Integer read FPort write FPort default 80;
      property Server: string read FServer write FServer;
   end;

{ Forward declaration }
   TIEDownload = class;

   TBSCB = class(TObject,
         IUnknown,
         IBindStatusCallback,
         IHttpNegotiate,
         IAuthenticate,
         IHTTPSecurity)
   private
      OutputFile: TFileStream;
      FBSCBTimer: TTimer;
      FStartTime, FInitializeTime: TDatetime;
      FTimedOut: Boolean;
      FBindCtx: IBindCtx;
      m_pPrevBSCB: IBindStatusCallback;
      FStream: IStream;
      FMoniker: IMoniker;
      FSender: TIEDownload;
      FRedirect: Boolean;
      FGlobalData: HGLOBAL;
      FDataSize: Integer;
      FTotalRead: Cardinal;
      FDataAvailable: Integer;
      StartTick: Int64;
      {$IFDEF DELPHI_7_UP}
      FormatSettings: TFormatSettings;
      {$ENDIF}
      FState: TState;
{IBindStatusCallBack methods}
      function OnLowResource(reserved: DWORD): HResult; stdcall;
      function OnProgress(ulProgress, ulProgressMax, ulStatusCode: ULONG;
         szStatusText: LPCWSTR): HResult; stdcall;
      function OnStartBinding(dwReserved: DWORD; pib: IBinding): HResult; stdcall;
      function OnStopBinding(hresult: HResult; szError: LPCWSTR): HResult; stdcall;
      function OnDataAvailable(grfBSCF: DWORD; dwSize: DWORD; formatetc: PFormatEtc;
         stgmed: PStgMedium): HResult; stdcall;
      function OnObjectAvailable(const iid: TGUID; punk: IUnknown): HResult; stdcall;
      function GetPriority(out nPriority): HResult; stdcall;
      function GetBindInfo(out grfBINDF: DWORD; var bindinfo: TBindInfo): HResult; stdcall;
{IHTTPNegotiate methods}
      function OnResponse(dwResponseCode: DWORD; szResponseHeaders, szRequestHeaders: LPCWSTR;
         out szAdditionalRequestHeaders: LPWSTR): HResult; stdcall;
      function BeginningTransaction(szURL, szHeaders: LPCWSTR; dwReserved: DWORD;
         out szAdditionalHeaders: LPWSTR): HResult; stdcall;
{IAuthenticate methods}
      function Authenticate(var hwnd: HWnd; var szUserName, szPassword: LPWSTR): HResult; stdcall;
{IWindowForBindingUI methods}
      function GetWindow(const GUIDReason: TGUID; out hwnd): HResult; stdcall;
{IHttpSecurity methods}
      function OnSecurityProblem(dwProblem: DWORD): HResult; stdcall;
      procedure Initialize;
      function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
      function _AddRef: Integer; stdcall;
      function _Release: Integer; stdcall;
      function Download: HRESULT;
      function DownloadCtx: HRESULT;
      procedure ClearAll;
      procedure TimerExpired(Sender: TObject);
      procedure UpdateEvents(Code: Integer; BusyState: Boolean; Sender: TBSCB;
         ulProgress, ulProgressMax: ULONG; szStatusText: LPCWSTR; Downloaded, ElapsedTime,
         Speed, RemainingTime, Status: string; Stream: TStream);
      function CheckCancelState: Integer;
      function GetContentDispFilename(): HRESULT;
      function CrackUrl(szUrl: WideString): WideString;
      procedure SetFileName(FileName: String);
   public
      Url: PWideChar;
      UrlComponents: TUrlComponents;
      UrlFileName: WideString;
      Stream: TStream;
      Binding: IBinding;
      ResponseCode: Cardinal;
      Info: TInfo;
      function GetSaveFileAs: WideString;
      constructor Create(DownloadInfo: TInfo); overload;
      constructor Create(DownloadInfo: TInfo; pmk: IMoniker; pbc: IBindCtx); overload;
      destructor Destroy; override;
      function QueryInfo(dwOption: DWORD; var Info: TDateTime): Boolean; overload;
      function QueryInfo(dwOption: DWORD; var Info: Cardinal): Boolean; overload;
      function QueryInfo(dwOption: DWORD; var info: string): Boolean; overload;
      function GetBindResult(out clsidProtocol: TCLSID; out dwResult: DWORD;
         out szResult: POLEStr): HResult;
      property FileName: String read Info.FileName write SetFileName;
      property State: TState read FState;
   end;

   TOnProgressEvent = procedure(Sender: TBSCB; ulProgress, ulProgressMax, ulStatusCode: ULONG;
      szStatusText: LPCWSTR; Downloaded, ElapsedTime, Speed, RemainingTime, Status: string) of object;
   TAuthenticateEvent = function(Sender: TBSCB; var hwnd: HWnd; var szUserName, szPassword: LPWSTR): HResult of object;
   TBeginningTransactionEvent = function(Sender: TBSCB; szURL, szHeaders: LPCWSTR; dwReserved: DWORD;
      out szAdditionalHeaders: LPWSTR): HResult of object;
   TOnResponseEvent = function(Sender: TBSCB; dwResponseCode: DWORD; szResponseHeaders, szRequestHeaders: LPCWSTR;
      out szAdditionalRequestHeaders: LPWSTR): HResult of object;
   TOnDataEvent = procedure(Sender: TBSCB; var Buffer: PByte; var BufLength: Cardinal) of object;
   TOnCompleteEvent = procedure(Sender: TBSCB; Stream: TStream; Result: HResult) of object;
   TOnResumeEvent = procedure(Sender: TBSCB; FileName: string; var Action: DWORD) of object;
   TGetWindowEvent = function(Sender: TBSCB; const GUIDReason: TGUID; out hwnd: LongWord): HResult of object;
   TOnSecurityProblemEvent = function(Sender: TBSCB; dwProblem: DWORD): HResult of object;
   TOnBindingEvent = procedure(var Sender: TBSCB; var Cancel: Boolean) of object;
   TOnErrorTextEvent = procedure(Sender: TBSCB; Text: string) of object;
   TOnStatusTextEvent = procedure(Sender: TBSCB; Text: string) of object;
   TOnRespondTextEvent = procedure(Sender: TBSCB; Text: string) of object;

   TUrlEncodeOption = (PostData, ExtraInfo);
   TUrlEncodeOptions = set of TUrlEncodeOption;

   TBindInfoOption = (
      Asynchronous, AsyncStorage, NoProgressiveRendering, OfflineOperation, GetNewestVersion,
      NoWriteCache, NeedFile, PullData, IgnoreSecurityProblem, Resynchronize, Hyperlink,
      No_UI, SilentOperation, Pragma_No_Cache, GetClassObject, Reserved_1, Free_Threaded,
      Direct_Read, Forms_Submit, GetFromCache_If_Net_Fail, FromUrlmon, Fwd_Back, Reserved_2,
      Reserved_3);

   TBindInfoOptions = set of TBindInfoOption;

   TCodePageOption = (
      Ansi, // default to ANSI code page
      OEM, // // default to OEM  code page
      Mac, // default to MAC  code page
      ThreadsAnsi, // Current thread's ANSI code page
      Symbol, // Symbol code page (42)
      UTF7, // Translate using UTF-7
      UTF8); // Translate using UTF-8

   TSecurity = class(TPersistent)
   private
      FInheritHandle: Boolean;
      FDescriptor: string;
   published
      property InheritHandle: boolean read FInheritHandle write FInheritHandle;
      property Descriptor: string read FDescriptor write FDescriptor;
   end;

   TRange = class(TPersistent)
   private
      FRangeBegin: Integer;
      FRangeEnd: Integer;
   published
      property RangeBegin: Integer read FRangeBegin write FRangeBegin;
      property RangeEnd: Integer read FRangeEnd write FRangeEnd;
   end;

   TBindVerbOption = (Get, Post, Put, Custom);
   TDownloadMethod = (dlStream, dlFile, dlBoth);

   TIEDownload = class(TComponent)
   private
      FDownloadMethod: TDownloadMethod;
      FObjList: TObjectList;
      FTimeOut: Integer;
      FSecurity: TSecurity;
      FRange: TRange;
      FRefCount: Integer;
      FBusy: Boolean;
      FOnBusy: TNotifyEvent;
      FullUserAgent: string;
      FURL: WideString;
      FDefaultProtocol: string;
      FCancel: Boolean;
      FOnResume: TOnResumeEvent;
      FOnData: TOnDataEvent;
      FPostData: string;
      FPutFileName: string;
      FAdditionalHeader: TStrings;
      FPassword: string;
      FUserName: string;
      FExtraInfo: string;
      FCustomVerb: string;
      FUserAgent: string;
      FDownloadDir: string;
      FBindVerbValue: Cardinal;
      FBindInfoValue: Cardinal;
      FCodePageValue: Cardinal;
      FBindInfoOptions: TBindInfoOptions;
      FBindVerbOption: TBindVerbOption;
      FCodePageOption: TCodePageOption;
      FUrlEncodeOptions: TUrlEncodeOptions;
      FUrlEncodeValue: Cardinal;
      FOnBinding: TOnBindingEvent;
      FBeginningtransAction: TBeginningTransActionEvent;
      FOnAuthenticate: TAuthenticateEvent;
      FOnResponse: TOnResponseEvent;
      FOnProgress: TOnProgressEvent;
      FOnComplete: TOnCompleteEvent;
      FGetWindow: TGetWindowEvent;
      FOnSecurityProblem: TOnSecurityProblemEvent;
      FOnErrorText: TOnErrorTextEvent;
      FProxySettings: TProxySettings;
      FOnStatusText: TOnStatusTextEvent;
      FOnRespondText: TOnRespondTextEvent;
      procedure SetAdditionalHeader(const Value: Tstrings);
      function AddPrefixHttp: WideString;
      procedure UpdateUserAgent;
      function SetDownloadDir: Boolean;
   protected
      procedure SetDefaultProtocol(const Value: string);
      procedure BusyChange; dynamic;
      procedure SetBindInfoOptions(const Value: TBindInfoOptions);
      procedure SetBindVerbOption(const Value: TBindVerbOption);
      procedure SetCodePageOption(const Value: TCodePageOption);
      procedure UpdateUrlEncodeValue;
      procedure SetUrlEncodeOptions(const Value: TUrlEncodeOptions);
      procedure SetDownloadMethod(const Value: TDownloadMethod);
      procedure FillInfo(var Info: TInfo);
      procedure UpdateBindInfoValue;
   public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure Loaded; override;
      procedure Go(Url: WideString);
      procedure Cancel(Item: TBSCB); overload;
      procedure Cancel; overload;
      property Busy: Boolean read FBusy write FBusy default False;
      procedure Download(
         pmk: IMoniker; // Identifies the object to be downloaded
         pbc: IBindCtx // Stores information used by the moniker to bind
         );
   published
      property DownloadMethod: TDownloadMethod read FDownloadMethod write SetDownloadMethod default dlStream;
      property TimeOut: Integer read FTimeOut write FTimeOut;
      property ExtraInfo: string read FExtraInfo write FExtraInfo;
      property CustomVerb: string read FCustomVerb write FCustomVerb;
      property CodePage: TCodepageOption read FCodepageOption write SetCodepageOption default Ansi;
      property DefaultProtocol: string read FDefaultProtocol write SetDefaultProtocol;
      property Method: TBindVerbOption read FBindVerbOption write SetBindVerbOption default Get;
      property Options: TBindInfoOptions read FBindInfoOptions write SetBindInfoOptions default
         [Asynchronous, AsyncStorage, PullData, NoWriteCache, GetNewestVersion];
      property UrlEncode: TUrlEncodeOptions read FUrlEncodeOptions write FUrlEncodeOptions;
      property Security: TSecurity read FSecurity write FSecurity;
      property Range: TRange read FRange write FRange;
      property Password: string read FPassword write FPassword;
      property UserName: string read FUserName write FUserName;
      property PutFileName: string read FPutFileName write FPutFileName;
      property AdditionalHeader: Tstrings read FAdditionalHeader write SetAdditionalHeader;
      property PostData: string read FPostData write FPostData;
      property UserAgent: string read FUserAgent write FUserAgent;
      property DownloadDir: string read FDownloadDir write FDownloadDir;
      property OnBinding: TOnBindingEvent read FOnBinding write FOnBinding;
      property OnBusyStateChange: TNotifyEvent read FOnBusy write FOnBusy;
      property OnErrorText: TOnErrorTextEvent read FOnErrorText write FOnErrorText;
      property OnRespondText: TOnRespondTextEvent read FOnRespondText write FOnRespondText;
      property OnStatusText: TOnStatusTextEvent read FOnStatusText write FOnStatusText;
      property OnProgress: TOnProgressEvent read FOnProgress write FOnProgress;
      property OnResume: TOnresumeEvent read FOnResume write FOnResume;
      property OnBeginningTransaction: TBeginningTransActionEvent read FBeginningTransAction write FBeginningTransAction;
      property OnResponse: TOnResponseEvent read FOnResponse write FOnResponse;
      property OnAuthenticate: TAuthenticateEvent read FOnAuthenticate write FOnAuthenticate;
      property OnComplete: TOnCompleteEvent read FOnComplete write FOnComplete;
      property OnData: TOnDataEvent read FOnData write FOnData;
      property OnGetWindow: TGetWindowEvent read FGetwindow write FGetwindow;
      property OnSecurityProblem: TOnSecurityProblemEvent read FOnSecurityProblem write FOnSecurityProblem;
      property URL: WideString read FURL write FURL;
      property ProxySettings: TProxySettings read FProxySettings write FProxySettings;

   end;

function BindStatusText(ulStatusCode: ULONG): string;
function ErrorText(errorCode: Integer): string;
function ResponseCodeText(ResponseCode: Integer): string;
function GetDownloadDirectory: WideString;
//function GetTempDir: string;

implementation

uses
   Registry, Forms{$IFDEF DELPHI_6_UP}, StrUtils{$ENDIF};

function ResponseCodeText(ResponseCode: Integer): string;
begin
   case ResponseCode of
      0: Result := 'OK';
      100: Result := 'Continue';
      101: Result := 'Switching Protocols';
      200: Result := 'OK';
      201: Result := 'Created';
      202: Result := 'Accepted';
      203: Result := 'Non-Authoritative Information';
      204: Result := 'No Content';
      205: Result := 'Reset Content';
      206: Result := 'Partial Content';
      300: Result := 'Multiple Choices';
      301: Result := 'Moved Permanently';
      302: Result := 'Moved Temporarily';
      303: Result := 'See Other';
      304: Result := 'Not Modified';
      305: Result := 'Use Proxy';
      400: Result := 'Bad Request';
      401: Result := 'Unauthorized';
      402: Result := 'Payment Required';
      403: Result := 'Forbidden';
      404: Result := 'Not Found';
      405: Result := 'Method Not Allowed';
      406: Result := 'None Acceptable';
      407: Result := 'Proxy Authentication Required';
      408: Result := 'Request Timeout';
      409: Result := 'Conflict';
      410: Result := 'Gone';
      411: Result := 'Length Required';
      412: Result := 'Precondition Failed';
      413: Result := 'Request Entity Too Large';
      414: Result := 'Request-URI Too Long';
      415: Result := 'Unsupported Media Type';
      416: Result := 'Requested Range Not Satisfiable';
      417: Result := 'Expectation Failed';
      500: Result := 'Internal Server Error';
      501: Result := 'Not Implemented';
      502: Result := 'Bad Gateway';
      503: Result := 'Service Unavailable';
      504: Result := 'Gateway Timeout';
      505: Result := 'HTTP Version Not Supported';
      else
         begin
            Result := 'Unknown Response: ' + IntToStr(ResponseCode);
         end;
   end
end;

function BindStatusText(ulStatusCode: ULONG): string;
begin
   case ulStatusCode of
      0: Result := 'OK';
      BINDSTATUS_REDIRECTING: Result := 'Server redirecting client...';
      BINDSTATUS_FINDINGRESOURCE: Result := 'Finding resource...';
      BINDSTATUS_CONNECTING: Result := 'Connecting...';
      BINDSTATUS_BEGINDOWNLOADDATA: Result := 'Beginning to download data...';
      BINDSTATUS_DOWNLOADINGDATA: Result := 'Downloading data...';
      BINDSTATUS_ENDDOWNLOADDATA: Result := 'Ending data download...';
      BINDSTATUS_BEGINDOWNLOADCOMPONENTS: Result := 'Beginning to download components...';
      BINDSTATUS_INSTALLINGCOMPONENTS: Result := 'Installing components...';
      BINDSTATUS_ENDDOWNLOADCOMPONENTS: Result := 'Ending component download...';
      BINDSTATUS_USINGCACHEDCOPY: Result := 'Using cached copy...';
      BINDSTATUS_SENDINGREQUEST: Result := 'Sending request...';
      BINDSTATUS_CLASSIDAVAILABLE: Result := 'CLSID available...';
      BINDSTATUS_MIMETYPEAVAILABLE: Result := 'MIME type available...';
      BINDSTATUS_CACHEFileNameAVAILABLE: Result := 'Cache file name available...';
      BINDSTATUS_BEGINSYNCOPERATION: Result := 'Synchronous operation has started...';
      BINDSTATUS_ENDSYNCOPERATION: Result := 'Ending synchronous operation...';
      BINDSTATUS_BEGINUPLOADDATA: Result := 'Beginning to upload data...';
      BINDSTATUS_UPLOADINGDATA: Result := 'Uploading data...';
      BINDSTATUS_ENDUPLOADDATA: Result := 'Ending data upload...';
      BINDSTATUS_PROTOCOLCLASSID: Result := 'Protocol handler CLSID is available...';
      BINDSTATUS_ENCODING: Result := 'Encoding data...';
      BINDSTATUS_VERIFIEDMIMETYPEAVAILABLE: Result := 'Verified MIME type is available...';
      BINDSTATUS_CLASSINSTALLLOCATION: Result := 'Class install location is available...';
      BINDSTATUS_DECODING: Result := 'Decoding data...';
      BINDSTATUS_LOADINGMIMEHANDLER: Result := 'Pluggable MIME handler is being loaded...';
      BINDSTATUS_CONTENTDISPOSITIONATTACH: Result := 'Content-Disposition resource is an attachment...';
      BINDSTATUS_FILTERREPORTMIMETYPE: Result := 'New MIME type available for resource...';
      BINDSTATUS_CLSIDCANINSTANTIATE: Result := 'CLSID to return on BindToObject...';
      BINDSTATUS_IUNKNOWNAVAILABLE: Result := 'IUnknown interface has been released...';
      BINDSTATUS_DIRECTBIND: Result := 'Connected directly to the pluggable protocol handler...';
      BINDSTATUS_RAWMIMETYPE: Result := 'Raw MIME type...';
      BINDSTATUS_PROXYDETECTING: Result := 'Proxy server has been detected...';
      BINDSTATUS_ACCEPTRANGES: Result := 'Valid types of range requests...';
      else
         Result := 'Unknown Bindstatus: ' + IntToStr(ulStatusCode);
   end;
end;

function ErrorText(ErrorCode: Integer): string;
begin
   case ErrorCode of
      S_OK: Result := 'OK';
      E_INVALIDARG: result := 'One or more parameters are invalid.';
      E_OUTOFMEMORY: Result := 'There was insufficient memory ...';
      E_ABORT: Result := 'Cancelled ...';
      STG_E_ACCESSDENIED: Result := 'Unable to access the storage object.';
      MK_E_NOSTORAGE: Result := 'The object identified by the moniker does not have its own storage.';
      MK_E_EXCEEDEDDEADLINE: Result := 'The operation could not be completed within the time limit specified by the bind context''s BIND_OPTS structure.';
      MK_E_CONNECTMANUALLY: Result := 'The operation was unable to connect to the storage, possibly because a network device could not be connected to. For more information, see IMoniker::BindToObject.';
      MK_E_INTERMEDIATEINTERFACENOTSUPPORTED: Result := 'An intermediate object was found but it did not support an interface required for an operation. For example, an item moniker returns this value if its container does not support the IOleItemContainer interface.';
      MK_E_SYNTAX: Result := 'A moniker could not be created because of invalid URL syntax.';
      INET_E_AUTHENTICATION_REQUIRED: Result := 'Authentication is needed to access the object.';
      INET_E_CANNOT_CONNECT: Result := 'The attempt to connect to the Internet has failed.';
      INET_E_CANNOT_INSTANTIATE_OBJECT: Result := 'CoCreateInstance failed.';
      INET_E_CANNOT_LOAD_DATA: Result := 'The object could not be loaded.';
      INET_E_CANNOT_LOCK_REQUEST: Result := 'The requested resource could not be locked.';
      INET_E_CONNECTION_TIMEOUT: Result := 'The Internet connection has timed out.';
      INET_E_DATA_NOT_AVAILABLE: Result := 'An Internet connection was established, but the data cannot be retrieved.';
      INET_E_DEFAULT_ACTION: Result := 'Use the default action.';
      INET_E_DOWNLOAD_FAILURE: Result := 'The download has failed (the connection was interrupted).';
      INET_E_INVALID_REQUEST: Result := 'The request was invalid.';
      INET_E_INVALID_URL: Result := 'The URL could not be parsed.';
      INET_E_NO_SESSION: Result := 'No Internet session was established.';
      INET_E_NO_VALID_MEDIA: Result := 'The object is not in one of the acceptable MIME types.';
      INET_E_OBJECT_NOT_FOUND: Result := 'The object was not found.';
      INET_E_QUERYOPTION_UNKNOWN: Result := 'The requested option is unknown.';
      INET_E_REDIRECT_TO_DIR: Result := 'The request is being redirected to a directory.';
      INET_E_REDIRECTING: Result := 'The request is being redirected.';
      INET_E_RESOURCE_NOT_FOUND: Result := 'The server or proxy was not found.';
      INET_E_SECURITY_PROBLEM: Result := 'A security problem was encountered.';
      INET_E_UNKNOWN_PROTOCOL: Result := 'The protocol is not known and no pluggable protocols have been entered that match.';
      INET_E_USE_EXTEND_Binding: Result := 'Reissue request with extended Binding.';
      INET_E_TERMINATED_BIND: Result := 'Binding is terminated.';
      INET_E_CODE_DOWNLOAD_DECLINED: Result := 'Permission to download is declined.';
      INET_E_RESULT_DISPATCHED: Result := 'Result is dispatched.';
      INET_E_CANNOT_REPLACE_SFP_FILE: Result := 'Cannot replace a protected System File Protection (SFP) file.';
      ERROR_INTERNET_SEC_CERT_DATE_INVALID: Result := 'SSL certificate date that was received from the server is bad. The certificate is expired.';
      ERROR_INTERNET_SEC_CERT_CN_INVALID: Result := 'SSL certificate common name (host name field) is incorrect???for example, if you entered www.server.com and the common name on the certificate says www.different.com.';
      ERROR_INTERNET_MIXED_SECURITY: Result := 'The content is not entirely secure. Some of the content being viewed may have come from unsecured servers.';
      ERROR_INTERNET_CHG_POST_IS_NON_SECURE: Result := 'The application is posting and attempting to change multiple lines of text on a server that is not secure.';
      ERROR_INTERNET_POST_IS_NON_SECURE: Result := 'The application is posting data to a server that is not secure.';
      ERROR_INTERNET_CLIENT_AUTH_CERT_NEEDED: Result := 'The server is requesting client authentication.';
      ERROR_INTERNET_INVALID_CA: Result := 'The function is unfamiliar with the Certificate Authority that generated the server''s certificate.';
      ERROR_INTERNET_CLIENT_AUTH_NOT_SETUP: Result := 'Client authorization is not set up on this computer.';
      ERROR_INTERNET_SEC_CERT_REV_FAILED: Result := 'SSL Certificate revision failed.';
      ERROR_INTERNET_SEC_CERT_REVOKED: Result := 'SSL certificate was revoked.';
      ERROR_INTERNET_FORTEZZA_LOGIN_NEEDED: Result := 'The requested resource requires Fortezza authentication.';
      ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR: Result := 'The application is moving from a non-SSL to an SSL connection because of a redirect.';
      ERROR_INTERNET_HTTPS_TO_HTTP_ON_REDIR: Result := 'The application is moving from an SSL to an non-SSL connection because of a redirect.';
      ERROR_HTTP_REDIRECT_NEEDS_CONFIRMATION: Result := 'The redirection requires user confirmation.';
      ERROR_INTERNET_SEC_CERT_ERRORS: Result := 'The SSL certificate contains errors.';
      else
         if ErrorCode <> 0 then
            Result := 'Error: ' + IntToStr(ErrorCode) + ' ' + SysErrorMessage(ErrorCode);
   end;
end;

function FormatTickToTime(TickCount: Cardinal): string;
var
   Hours, Minutes, Seconds, MSecs: Cardinal;
   S_HUR, S_MIN, S_SEC, S_MSC: string;
begin
   S_MSC := '000';
   S_SEC := '00';
   S_MIN := '00';
   S_HUR := '00';
   try
      if (TickCount > 0) then
         begin
            MSecs := TickCount mod MSecsPerSec;
            TickCount := TickCount div MSecsPerSec;
            S_MSC := Format('%.3d', [MSecs]);
            Seconds := TickCount mod SecsPerMin;
            TickCount := TickCount div SecsPerMin;
            S_SEC := Format('%.2d', [Seconds]);
            Minutes := TickCount mod MinsPerHour;
            TickCount := TickCount div MinsPerHour;
            S_MIN := Format('%.2d', [Minutes]);
            Hours := TickCount mod HoursPerDay;
            S_HUR := Format('%.2d', [Hours]);
         end;
   finally
      if S_HUR = '00' then
         Result := S_MIN + 'm :' + S_SEC + 's :' + S_MSC + 'ms';
      if (S_HUR = '00') and (S_MIN = '00') then
         Result := S_SEC + 's :' + S_MSC + 'ms';
      if S_HUR <> '00' then
         Result := S_HUR + 'h :' + S_MIN + 'm :' + S_SEC + 's :' + S_MSC + 'ms';
   end;
end;

function FormatSize(Byte: Double): string;
begin
   if (Byte < 1024) then
      Result := Format('%.2n b', [Byte])
   else
      begin
         Byte := (Byte / 1024);
         if (Byte < 1024) then
            Result := Format('%.2n Kb', [Byte])
         else
            begin
               Byte := (Byte / 1024);
               Result := Format('%.2n Mb', [Byte]);
            end;
      end;
end;

function IsValidURL(const URL: Widestring): Boolean;
begin
   if UrlMon.IsValidURL(nil, PWideChar(URL), 0) = S_OK then
      Result := True
   else
      Result := False;
end;

function FileNameFromUrl(Url: WideString): WideString;
var
   i: Integer;
begin
   i := Length(Url);
   repeat
      Result := Url[i] + Result;
      Dec(i);
   until (url[i] = '/') or (i = 0);
   if AnsiPos('.', Result) = 0 then
      Result := 'index.html'
end;

// End of Accesories------------------------------------------------------------

procedure TBSCB.SetFileName(FileName: String);
begin
   if (Info.FileName <> FileName) then begin
      Info.FileName := FileName;
      Info.FileExt := ExtractFileExt(FileName);
      if (Length(Info.FileExt) > 0) then
         Delete(Info.FileExt, 1, 1);  // remove period character that separates the name and extension parts
   end;
end;

function TBSCB.GetWindow(const GUIDReason: TGUID; out hwnd): HResult;
begin
   if Assigned(FSender.FGetWindow) then
      Result :=
         FSender.FGetWindow(self, GUIDReason, LongWord(hwnd))
   else
      Result := S_OK;
end;

function TBSCB.OnSecurityProblem(dwProblem: DWORD): HResult;
begin
   UpdateEvents(dwProblem, False, Self, 0, 0, 'Security Problem.',
      '0', '0', '0', '0', 'Security Problem.', nil);
   if Assigned(FSender.FOnSecurityProblem) then
      begin
         Result := FSender.FOnSecurityProblem(Self, dwProblem);
      end
   else
      Result := S_OK;
end;

function TBSCB.Authenticate(var hwnd: HWnd; var szUserName, szPassword: LPWSTR): HResult;
var
   Len: Integer;
begin
{Optional Results  on Authenticate callback :
  S_OK  = Authentication was successful.
  E_ACCESSDENIED = Authentication failed.
  E_INVALIDARG = One or more parameters are invalid.}
   Result := S_OK;
   if CheckCancelState = E_Abort then
      Exit;
   if Assigned(FSender.FOnAuthenticate) then
      Result := FSender.FOnAuthentiCate(Self, hwnd, szUserName, szPassword)
   else
      begin
         if Info.UserName <> '' then
            begin
               Len := Length(Info.UserName);
               szUserName := CoTaskMemAlloc(Len * 2);
               if szUserName = nil then
                  begin
                     UpdateEvents(E_OUTOFMEMORY, False, Self, 0, 0, 'Out of memory.',
                        '0', '0', '0', '0', 'Out of memory.', nil);
                     Exit;
                  end;
               MultiByteToWideChar(0, 0, Pointer(Info.UserName), Len, szUserName, Len);
            end
         else
            szUserName := nil;
         if Info.Password <> '' then
            begin
               Len := Length(Info.Password);
               szPassword := CoTaskMemAlloc(Len * 2);
               if szPassword = nil then
                  begin
                     UpdateEvents(E_OUTOFMEMORY, False, Self, 0, 0, 'Out of memory.',
                        '0', '0', '0', '0', 'Out of memory.', nil);
                     Exit;
                  end;
               MultiByteToWideChar(0, 0, Pointer(Info.Password), Len, szPassword, Len);
            end
         else
            szPassword := nil;
      end;
   if szUserName <> nil then
      CoTaskMemFree(szUserName);
   if szUserName <> nil then
      CoTaskMemFree(szUserName);
end;

function TBSCB.BeginningTransaction(szURL, szHeaders: LPCWSTR; dwReserved: DWORD;
   out szAdditionalHeaders: LPWSTR): HResult;
var
   sr: TSearchRec;
   Action: Cardinal;
   S: string;
   Size: Longint;
   x, Len: Integer;
begin
   if CheckCancelState = E_Abort then
      begin
         Result := E_Abort;
         Exit;
      end;
   S := FSender.FullUserAgent + #13#10;
   if (Info.FileName <> '') then
      begin
         if FindFirst(Info.FileName, faAnyFile, sr) = 0 then //TODO: does the FileName contains the directory???
            begin
               Size := sr.Size;
               FindClose(sr);
               Info.RangeEnd := 0;
               Action := 0;
               if Assigned(FSender.OnResume) then
                  FSender.OnResume(Self, Info.FileName, Action);
               case Action of
                  RESUME_RESPONSE_CANCEL:
                     begin
                        Result := E_ABORT;
                        Exit;
                     end;
                  RESUME_RESPONSE_OVERWRITE: Info.RangeBegin := 0;
                  else
                     Info.RangeBegin := Size;
               end;
            end
         else
            begin
               Info.RangeBegin := 0;
               Info.RangeEnd := 0;
            end;
      end;
   if ((Info.RangeBegin <> 0) or (Info.RangeEnd <> 0)) then
      begin
         S := S + 'Range: bytes=' + IntToStr(Info.RangeBegin) + '-';
         if Info.RangeEnd <> 0 then
            S := S + IntToStr(Info.RangeEnd) + #13#10
         else
            S := S + #13#10;
      end;
   if (Info.AdditionalHeader[0] <> '') then
      for x := 0 to Info.AdditionalHeader.Count - 1 do
         S := S + info.additionalheader[x] + '#$D#$A"';
   Len := Length(S);
   szAdditionalHeaders := CoTaskMemAlloc(Len * 2);
   MultiByteToWideChar(0, 0, Pointer(S), Len, szAdditionalHeaders, Len);
   if Assigned(FSender.FBeginningTransaction) then
      Result := FSender.FBeginningTransaction(Self, szUrl, szHeaders, dwReserved, szAdditionalHeaders)
   else
      Result := S_OK;
   FBSCBTimer.Enabled := True;
   FTimedOut := False;
end;

function TBSCB.OnResponse(dwResponseCode: DWORD; szResponseHeaders,
   szRequestHeaders: LPCWSTR; out szAdditionalRequestHeaders: LPWSTR): HResult;
var
   Len: Cardinal;
   s: string;
   Dest: WideString;
begin
   if CheckCancelState = E_Abort then
      begin
         Result := E_Abort;
         //OnStopBinding will called automatically, so no Binding.Abort is necessary
         Exit;
      end;
   FSender.SetDownloadDir;
   case FSender.FDownloadMethod of
      dlStream: Dest := '';
      dlFile: Dest := FSender.FDownloadDir;
      dlBoth: Dest := FSender.FDownloadDir;
   end;
   ResponseCode := dwResponseCode;
   if (ResponseCode >= 300) then
      UpdateEvents(ResponseCode, False, Self, 0, 0, 'Error.',
         '0', '0', '0', '0', 'Error.', nil);

   if (QueryInfo(HTTP_QUERY_CUSTOM, Len) and (Len = 0)) or
      (QueryInfo(HTTP_QUERY_CONTENT_LENGTH, Len) and (Len = 0)) then
      begin
         Result := E_ABORT;
         UpdateEvents(Result, False, Self, 0, 0, 'File Size Error.',
            '0', '0', '0', '0', 'File Size Error.', nil);
      end
   else
      if (ResponseCode >= 400) and (ResponseCode < 500) then
         begin
            Result := E_ABORT;
            UpdateEvents(Result, False, Self, 0, 0, 'Aborted.',
               '0', '0', '0', '0', ResponseCodeText(ResponseCode), nil);
         end
      else
         begin
            if Assigned(FSender.FOnResponse) then
               Result := FSender.FOnResponse(self, dwResponseCode,
                  szResponseHeaders, szRequestHeaders, szAdditionalRequestHeaders)
            else
               Result := S_OK;
            if (Info.RangeBegin <> 0) and (Info.FileName <> '') then
               begin
                  QueryInfo(HTTP_QUERY_ACCEPT_RANGES, S);
                  if (S = 'bytes') or (dwResponseCode = 206) then  //'Partial Content'
                     begin
                        OutputFile := TFileStream.Create(GetSaveFileAs(), fmOpenReadWrite);
                        OutputFile.Seek(0, soFromEnd);
                     end
                  else
                     begin
                        OutputFile := TFileStream.Create(GetSaveFileAs(), fmCreate);
                        Info.RangeBegin := 0;
                     end;
               end
            else
            begin
               if Info.FileName <> '' then
                  OutputFile := TFileStream.Create(GetSaveFileAs(), fmCreate);
            end
         end;
end;

function TBSCB.GetBindInfo(out grfBINDF: DWORD; var bindinfo: TBindInfo): HResult;
var
   PutFile: TFileStream;
   Len: Integer;
begin
   grfBINDF := Info.BindInfoValue;
   with BindInfo do
      begin
         cbSize := sizeof(TBINDINFO);
         if FReDirect then
            dwBindVerb := BINDVERB_GET
         else
            dwBindVerb := Info.BindVerbValue;
         grfBindInfoF := Info.UrlEncodeValue; //I don't think it is supported by Urlmon.dll yet
         dwCodePage := Info.CodepageValue;
         with SecurityAttributes do
            begin
               nLength := SizeOf(TSecurityAttributes);
               bInheritHandle := Info.InheritHandle;
               if Info.Descriptor <> '' then
                  lpSecurityDescriptor := PChar(Info.Descriptor)
               else
                  lpSecurityDescriptor := nil;
            end;
         if Info.ExtraInfo <> '' then
            begin
               Len := Length(Info.ExtraInfo);
               szExtraInfo := CoTaskMemAlloc(Len * 2);
               MultiByteToWideChar(0, 0, Pointer(Info.ExtraInfo), Len, szExtraInfo, Len);
            end
         else
            szExtraInfo := nil;
         if (Info.BindVerbValue = BINDVERB_PUT) then
            begin
               PutFile := TFileStream.Create(Info.PutFileName, fmOpenRead);
               PutFile.Seek(0, 0);
               FglobalData := GlobalAlloc(GPTR, PutFile.Size);
               FDataSize := PutFile.Size;
               PutFile.ReadBuffer(Pointer(FGlobalData)^, PutFile.Size);
               PutFile.Free;
            end;
         if (Info.BindVerbValue = BINDVERB_POST) then
            begin
               FglobalData := GlobalAlloc(GPTR, Length(Info.Postdata) + 1);
               FDataSize := Length(Info.PostData) + 1;
               Move(Info.Postdata[1], Pointer(FGlobalData)^, Length(Info.Postdata));
            end;
         if (Info.CustomVerb <> '') and (dwBindVerb = BINDVERB_CUSTOM) then
            begin
               Len := Length(Info.CustomVerb);
               szCustomVerb := CoTaskMemAlloc(Len * 2);
               MultiByteToWideChar(0, 0, Pointer(Info.CustomVerb), Len, szCustomVerb, Len);
            end
         else
            szCustomVerb := nil;
         Fillchar(stgmedData, 0, sizeof(STGMEDIUM));
         cbStgmedData := FDataSize;
         with StgmedData do
            begin
               if dwBindVerb = BINDVERB_GET
                  then
                  Tymed := TYMED_NULL
               else
                  Tymed := TYMED_HGLOBAL; // this is the only medium urlmon supports right now
               hGlobal := FGlobalData;
               IUnknown(unkForRelease) := Self;
            end;
      end;
   Result := S_OK;
end;

function TBSCB.GetPriority(out nPriority): HResult;
begin
   Result := CheckCancelState;
end;

function TBSCB.OnDataAvailable(grfBSCF, dwSize: DWORD; formatetc: PFormatEtc;
   stgmed: PStgMedium): HResult;
var
   Data: PByte;
   BufL, dwRead, dwActuallyRead: Cardinal;
begin
   if FStartTime = 0 then
      FStartTime := Now;
   if CheckCancelState = E_Abort then
      begin
         Result := E_Abort;
         Exit;
      end
   else
      begin
         if Assigned(FBSCBTimer) then
            begin
               FBSCBTimer.Enabled := False;
               FBSCBTimer.Enabled := True;
            end;
         if (grfBSCF = grfBSCF or BSCF_FIRSTDATANOTIFICATION) then
            begin
               if (FStream = nil) and (stgmed.tymed = TYMED_ISTREAM) then
                  FStream := IStream(stgmed.stm);
               if Assigned(m_pPrevBSCB) and not Assigned(Outputfile) and (Info.FileName <> '') then
                  try
    //TODO: check for resume
                     Outputfile := TFileStream.Create(GetSaveFileAs(), fmCreate);
                     Info.RangeBegin := 0;
                  except on EFCreateError do
                        begin
                           Binding.Abort;
                           result := E_FAIL;
                           Exit;
                        end;
                  end;
            end;
         dwRead := dwSize - FTotalRead;
         dwActuallyRead := 0;
         if (dwRead > 0) then
            repeat
               Data := AllocMem(dwRead + 1);
               FStream.Read(Data, dwRead, @dwActuallyRead);
               bufl := dwActuallyRead;
               if Assigned(FSender.FOnData) then
                  begin
                     FSender.FOnData(self, Data, bufl);
                  end;
               if (Info.FileName <> '') and Assigned(Outputfile) then
                  OutPutfile.WriteBuffer(Data^, bufl)
               else
                  if Assigned(Stream) then
                     Stream.WriteBuffer(Data^, bufl);
               Inc(FTotalRead, dwActuallyRead);
               FreeMem(Data);
            until dwActuallyRead = 0;
      end;
   Result := S_OK;
end;

function TBSCB.OnLowResource(reserved: DWORD): HResult;
begin
   Result := S_OK;
end;

function TBSCB.OnObjectAvailable(const iid: TGUID;
   punk: IUnknown): HResult;
begin
   Result := S_OK;
end;

function TBSCB.GetContentDispFilename(): HRESULT;
const
   CD_FILE_PARAM = 'filename=';
var
   i: Integer;
   sTmp: string;
   res: Boolean;
begin
   result := E_FAIL;
   res := QueryInfo(HTTP_QUERY_CONTENT_DISPOSITION, sTmp);
   if not res then
      Exit;
 // Content-Disposition: attachment; filename=genome.jpeg; modification-date="Wed, 12 Feb 1997 16:29:51 -0500";
   i := Pos(CD_FILE_PARAM, sTmp);
   if (i > 0) then
      begin
         sTmp := Copy(sTmp, i + Length(CD_FILE_PARAM), Length(sTmp) - i);
         if (sTmp[1] = '"') then
            i := Pos('";', sTmp)
         else
            i := Pos(';', sTmp);
  //TODO: what's happen, if the filename contains a quotion mark?
         if (i > 0) then
            sTmp := Copy(sTmp, 1, i);
         if (sTmp[1] = '"') then
            FileName := Copy(sTmp, 2, Length(sTmp) - 2)
         else
            FileName := sTmp;
         if (Length(sTmp) > 0) then
            Result := S_OK;
      end;

end;

function TBSCB.OnProgress(ulProgress, ulProgressMax, ulStatusCode: ULONG;
   szStatusText: LPCWSTR): HResult;
var
   Speed, Elapsed, Downloaded, RemainingTime, Status: string;
   _Speed: Single;
   currentTime: TDateTime;
begin
   if ulProgressMax = 0 then
      ulProgressMax := ulProgress * 100; // dummy value in case file size is unknown
   if CheckCancelState = E_Abort then
      begin
         Result := E_Abort;
         Exit;
      end
   else
      begin
         if (Assigned(FSender.FOnStatusText)) and (BindStatusText(ulStatusCode) <> 'Bindstatus Unknown') then
            FSender.FOnStatusText(Self, BindStatusText(ulStatusCode));
         Status := BindStatusText(ulStatusCode);
         currentTime := Now;
         Elapsed := TimeToStr(currentTime - FInitializeTime{$IFDEF DELPHI_7_UP}, FormatSettings{$ENDIF});
         if Assigned(m_pPrevBSCB) then
            begin
              //Need to do this otherwise a filedownload dlg will be displayed
              //as we are downloading the file.
               if (ulStatusCode = BINDSTATUS_CONTENTDISPOSITIONATTACH) then
                  begin
                     Result := S_OK;
                     Exit;
                  end;
               m_pPrevBSCB.OnProgress(ulProgress, ulProgressMax, ulStatusCode, szStatusText);
            end;

         case ulStatusCode of
            BINDSTATUS_ENDDOWNLOADDATA:
               begin
                 // FSender.SaveFileAs := Info.FileName;
                  Downloaded := 'Done.';
                  ulProgress := ulProgressMax;
                  Speed := '0/00  kb/sec';
                  RemainingTime := '00.00.00';
                  Status := 'Done.';
               end;
            BINDSTATUS_DOWNLOADINGDATA:
               if Assigned(FSender.FOnProgress) then
                  begin
                     try
                        if (currentTime - FStartTime) > 0 then
                           _Speed := (ulProgress - Info.RangeBegin) / ((currentTime - FStartTime) * SecsPerDay * MSecsPerSec)
                        else
                           _Speed := 0;

                        Speed := Format('%.1f kb/sec', [_Speed]{$IFDEF DELPHI_7_UP}, FormatSettings{$ENDIF});
                        if (ulProgressMax > 0) and ((_speed) > 0) then
                              RemainingTime := TimeToStr((ulProgressMax - ulProgress) /
                                 (_speed * 1000 * SecsPerDay){$IFDEF DELPHI_7_UP}, FormatSettings{$ENDIF})
                        else
                           RemainingTime := TimeToStr(0{$IFDEF DELPHI_7_UP}, FormatSettings{$ENDIF});
                        Downloaded := FormatSize(ulProgress + Info.RangeBegin);
                     except
                        on EZeroDivide do
                           RemainingTime := TimeToStr(0{$IFDEF DELPHI_7_UP}, FormatSettings{$ENDIF});
                     end;
                  end;
            BINDSTATUS_REDIRECTING:
               begin
                  FRedirect := (ulStatusCode = BINDSTATUS_REDIRECTING);
                  if (Assigned(FSender.FOnStatusText)) then
                     FSender.FOnStatusText(Self, BindStatusText(ulStatusCode));
               end
            else;
         end;
         if Assigned(FSender.FOnProgress) then
            FSender.FOnProgress(Self, ulProgress + Info.RangeBegin, ulProgressMax + Info.RangeBegin, ulStatusCode, szStatusText,
               Downloaded, Elapsed, Speed, RemainingTime, Status);
      end;
   Result := S_OK;
end;

function TBSCB.CheckCancelState: Integer;
begin
   if FSender.FCancel or (FState = Canceled) then
      begin
         Binding.Abort;
         Result := E_ABORT;
         UpdateEvents(Result, False, Self, 0, 0, 'Aborted.',
            '0', '0', '0', '0', 'Aborted.', nil);
      end
   else
      Result := S_OK;
end;

procedure TBSCB.TimerExpired(Sender: TObject);
begin
   FTimedOut := True;
end;

function TBSCB.OnStartBinding(dwReserved: DWORD; pib: IBinding): HResult;
var
   Cancel: Boolean;
begin
   if CheckCancelState = E_Abort then
      Result := E_FAIL
   else
      begin
         Binding := pib;
         Binding._AddRef;
         QueryInfo(HTTP_QUERY_CONTENT_LENGTH, Info.FileSize);
         GetContentDispFilename();
         if Assigned(m_pPrevBSCB) then
            m_pPrevBSCB.OnStopBinding(HTTP_STATUS_OK, nil);
         case FSender.FDownloadMethod of
            dlStream: FileName := '';
            dlFile, dlBoth:
               begin
                  if FileName = '' then
                     FileName := FileNameFromUrl(Url);
               end;
         end;
         if Assigned(FSender.FOnBinding) then
            begin
               Cancel := False;
               FSender.FOnBinding(Self, Cancel);
               if Cancel then
                  Result := E_FAIL
               else
                  Result := S_OK;
            end
         else begin
            FState := Downloading;
            Result := S_OK;
         end;
      end;
end;

function TBSCB.OnStopBinding(hresult: hResult; szError: LPCWSTR): HResult;
var
   clsidProtocol: TCLSID;
   dwResult: DWORD;
   szResult: POLEStr;
begin
   Result := S_OK;
   if (Assigned(m_pPrevBSCB) and Assigned(FBindCtx)) then
      begin
            //Register PrevBSCB and release our pointers
         FBindCtx.RegisterObjectParam('_BSCB_Holder_', m_pPrevBSCB);
         m_pPrevBSCB._Release();
         m_pPrevBSCB := nil;
         FBindCtx._Release();
         FBindCtx := nil;
            //Decrease our ref count, so when release is called
            //we delete this object
         dec(FSender.FRefCount);
      end;
   if (GetBindResult(clsidProtocol, dwResult, szResult) = S_OK) then begin
   end;
   if FTimedOut then
      hResult := INET_E_CONNECTION_TIMEOUT;
   if hResult = S_OK then begin
      FState := Finished;
      if (Info.FileName = '') then
         UpdateEvents(hResult, False, Self, 0, 0, 'Ready',
            '', '', '', '', 'Finished', Stream)
      else begin
         OutPutFile.Seek(0, 0);
         UpdateEvents(hResult, False, Self, 0, 0, 'Ready',
            '', '', '', '', 'Finished', OutPutFile);
      end;
   end else
      UpdateEvents(hResult, False, Self, 0, 0, 'Error',
         '', '', '', '', (ErrorText(hResult)), nil);

   if (Assigned(OutPutFile)) then begin
      OutPutFile.Free;
      OutPutFile := nil;
   end;
   if Info.BindInfoValue = Info.BindInfoValue or BINDF_ASYNCHRONOUS then
      FSender.FObjList.Extract(Self); // Remove caused the error

   Binding._Release;
   Binding := nil;

   ClearAll; // releases binding ...
    // call update complete only on real complete or abort
   if Assigned(FSender.FOnComplete) then
      FSender.FOnComplete(Self, Stream, hResult);
end;

function TBSCB.CrackUrl(szUrl: WideString): WideString;
// sets a new url and try to cracks it imediatally
var
   bRetVal: Boolean;
   dwCrackFlags: DWord;
   lpszTmp: PChar;
//	dwPos: DWord;
   i: Integer;
   dwLastError: DWord;
   szMsg: String;
begin
   if (szUrl = '') then
      Exit;
{	szUrl := LowerCase(szUrl);
 dwPos := Pos(':', szUrl);
 if ((dwPos = 0) or (dwPos > INTERNET_MAX_SCHEME_LENGTH)) and (Pos('\\', szUrl) = 0) then
  if (Pos('ftp.', szUrl) = 1) then
   szUrl := 'ftp://' + szUrl
  else szUrl := 'http://' + szUrl;
 if (Pos('cookie:', szUrl) = 1) then
  Insert('//', szUrl, Pos(':', szUrl) + 1);
}

   dwCrackFlags := (ICU_DECODE or ICU_ESCAPE);
   with UrlComponents do
      begin
         dwSchemeLength := INTERNET_MAX_SCHEME_LENGTH;
         dwHostNameLength := INTERNET_MAX_HOST_NAME_LENGTH;
         dwUserNameLength := INTERNET_MAX_USER_NAME_LENGTH;
         dwPasswordLength := INTERNET_MAX_Password_LENGTH;
         dwUrlPathLength := INTERNET_MAX_PATH_LENGTH;
         dwExtraInfoLength := INTERNET_MAX_URL_LENGTH;
      end;

   bRetVal := InternetCrackUrl(PChar(String(szUrl)),
      0,
      dwCrackFlags, UrlComponents);

   if bRetVal then
      begin
         with UrlComponents do
            begin
               if (nScheme = INTERNET_SCHEME_UNKNOWN) then
                  for i := 0 to High(EXT_SCHEMES) do
                     if (EXT_SCHEMES[i].Name = lpszScheme) then
                        begin
                           nScheme := EXT_SCHEMES[i].Id;
                           Break;
                        end;
               if (AnsiStrRScan(lpszUrlPath, '.') <> nil) then
                  begin
                     lpszTmp := AnsiStrRScan(lpszUrlPath, '/');
                     if (lpszTmp <> nil) then
                        UrlFileName := lpszTmp + 1
                     else
                        UrlFileName := lpszUrlPath;
                  end
               else
                  UrlFileName := '';
            end;
      end
      else begin
        dwLastError := GetLastError();
        szMsg := SysErrorMessage(dwLastError);
         OutputDebugString(PChar(szMsg));
      end;
         
   Result := UrlFileName;
end; { TUrlHandler.SetUrl }

procedure TBSCB.Initialize;
begin
   ClearAll;
   FState := Requested;
   StartTick := GetTickCount;
   FInitializeTime := Now;
   FBSCBTimer := TTimer.Create(nil);
   FBSCBTimer.OnTimer := TimerExpired;
   FBSCBTimer.Interval := Info.TimeOut;
   FTimedOut := False;
   FSender := TIEDownload(Info.Sender);
   {$IFDEF DELPHI_7_UP}
   GetLocaleFormatSettings(LOCALE_USER_DEFAULT, FormatSettings);
//   FormatSettings.LongTimeFormat := 'mm:ss.zzz';
   {$ENDIF}

   ZeroMemory(@UrlComponents, sizeof(UrlComponents));
   with UrlComponents do
      try
         dwStructSize := sizeof(UrlComponents);
         dwSchemeLength := INTERNET_MAX_SCHEME_LENGTH;
         dwHostNameLength := INTERNET_MAX_HOST_NAME_LENGTH;
         dwUserNameLength := INTERNET_MAX_USER_NAME_LENGTH;
         dwPasswordLength := INTERNET_MAX_Password_LENGTH;
         dwUrlPathLength := INTERNET_MAX_PATH_LENGTH;
         dwExtraInfoLength := INTERNET_MAX_URL_LENGTH;
         GetMem(lpszScheme, INTERNET_MAX_SCHEME_LENGTH + 1);
         GetMem(lpszHostName, INTERNET_MAX_HOST_NAME_LENGTH + 1);
         GetMem(lpszUserName, INTERNET_MAX_USER_NAME_LENGTH + 1);
         GetMem(lpszPassword, INTERNET_MAX_Password_LENGTH + 1);
         GetMem(lpszUrlPath, INTERNET_MAX_PATH_LENGTH + 1);
         GetMem(lpszExtraInfo, INTERNET_MAX_URL_LENGTH + 1);
      except
      end;
end;

constructor TBSCB.Create(DownloadInfo: TInfo);
begin
   inherited Create;
   Info := DownloadInfo;
   Initialize;
   Url := Info.Url;
   FileName := CrackUrl(Url);
   if (Info.FileName = '') then
      Stream := TMemoryStream.Create;
   UpdateEvents(S_OK, True, Self, 0, 0, 'Starting...',
      '', '', '', '', 'Starting...', nil);
   if CheckCancelState = E_Abort then
      Exit
   else
      Download()
end;

constructor TBSCB.Create(DownloadInfo: TInfo; pmk: IMoniker; pbc: IBindCtx);
begin
   inherited Create;
   Info := DownloadInfo;
   Initialize;
   FMoniker := pmk;
   FBindCtx := pbc;
   FMoniker.GetDisplayName(FBindCtx, nil, Url);
   Info.Url := Url;
   FileName := CrackUrl(Url);
    //if (Info.FileName = '') then
    //    Stream := TMemoryStream.Create;
   UpdateEvents(S_OK, True, Self, 0, 0, 'Starting...',
      '', '', '', '', 'Starting...', nil);
   if CheckCancelState = E_Abort then
      Exit
   else
      DownloadCtx;
end;

function TBSCB.DownloadCtx: HRESULT;
var
   hr: HRESULT;
   pPrevBSCB, tmp: IBindStatusCallback;
begin
   Result := S_FALSE;

   hr := RegisterBindStatusCallback(FBindCtx, self, pPrevBSCB, 0);
   if ((FAILED(hr)) and Assigned(pPrevBSCB)) then
      begin
         hr := FBindCtx.RevokeObjectParam('_BSCB_Holder_');
         if (SUCCEEDED(hr)) then
            begin
   //Attempt register again, should succeed now
               hr := RegisterBindStatusCallback(FBindCtx, self, tmp, 0);
               if (SUCCEEDED(hr)) then
                  begin
    //Need to pass a pointer for BindCtx and previous BSCB to our implementation
                     m_pPrevBSCB := pPrevBSCB;
                     self._AddRef();
                     m_pPrevBSCB._AddRef();
                     FBindCtx._AddRef();
                  end;
            end;
      end;
{   else
      begin
         if Assigned(FSender.FOnComplete) then
            FSender.FOnComplete(self, Stream, hr);
         exit;
      end;
}
   hr := FMoniker.BindToStorage(FBindCtx, nil, IStream, fstream);

   if (hr <> S_OK) and (hr <> MK_S_ASYNCHRONOUS) then
      begin
         if Assigned(FSender.FOnComplete) then
            FSender.FOnComplete(self, Stream, hr);
         Exit;
      end
   else
      hr := S_OK;

   result := hr;
end;

destructor TBSCB.Destroy;
begin
   with UrlComponents do
      try
         FreeMem(lpszScheme);
         FreeMem(lpszHostName);
         FreeMem(lpszUserName);
         FreeMem(lpszPassword);
         FreeMem(lpszUrlPath);
         FreeMem(lpszExtraInfo);
      except
      end;

   ClearAll;
   if Assigned(FBSCBTimer) then
      FreeAndNil(FBSCBTimer);
   if Assigned(OutPutFile) then
      FreeAndNil(OutPutFile);
   if Assigned(Stream) then
      FreeAndNil(Stream);
   Info.AdditionalHeader.Free;
   if (FGlobalData <> 0) then
      GlobalFree(FGlobalData);
   inherited;
end;

function TBSCB.Download: HRESULT;
var
   hr: HRESULT;
   OleStrUrl: POleStr;
begin
   Result := S_False;
   OleStrUrl := StringToOleStr(Info.URL);
   try
      hr := CreateURLMoniker(nil, OleStrUrl, FMoniker);
      if (hr <> S_OK) then
         begin
            UpdateEvents(HR, False, Self, 0, 0, 'Error creating Moniker.',
               '0', '0', '0', '0', 'Error creating Moniker.', nil);
            Exit;
         end;
      hr := CreateAsyncBindCtx(0, SELF, nil, FBindCtx);
      if (hr <> S_OK) then
         begin
            UpdateEvents(HR, False, Self, 0, 0, 'Error creating Async Bind.',
               '0', '0', '0', '0', 'Error creating Async Bind.', nil);
            Exit;
         end;
      hr := UrlMon.IsValidUrl(FBindCtx, OleStrUrl, 0);
      if hr <> S_OK then
         begin
            UpdateEvents(MK_E_SYNTAX, False, Self, 0, 0, 'Error in syntax.',
               '0', '0', '0', '0', 'Error in syntax.', nil);
            Exit;
         end;
      hr := FMoniker.BindToStorage(FbindCtx, nil, IStream, fstream);

      if (hr <> S_OK) and (hr <> MK_S_ASYNCHRONOUS) then
         begin
            UpdateEvents(MK_E_SYNTAX, False, Self, 0, 0, 'Error in Bind To Storage.',
               '0', '0', '0', '0', 'Error in Bind To Storage.', nil);
            Exit;
         end
      else
         begin
            Result := S_OK;
         end;
   finally
      SysFreeString(oleStrUrl);
   end;
end;

procedure TBSCB.ClearAll;
begin
   if Assigned(Binding) then
      Binding.Abort;
   FMoniker := nil;
   FBindCtx := nil;
   FGlobalData := 0;
   FTotalRead := 0;
   FDataAvailable := 0;
   FStartTime := 0;
   m_pPrevBSCB := nil;
end;

function TBSCB._AddRef: Integer;
begin
   Result := InterlockedIncrement(FSender.FRefCount);
end;

function TBSCB._Release: Integer;
begin
   Result := InterlockedDecrement(FSender.FRefCount);
   if Result = 0 then
      Destroy;
end;

function TBSCB.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
   if GetInterface(IID, Obj) then
      Result := 0
   else
      Result := E_NOINTERFACE;
end;

function TBSCB.QueryInfo(dwOption: DWORD; var Info: Cardinal): Boolean;
var
   HttpInfo: IWinInetHttpInfo;
   C: Cardinal;
   BufferLength: Cardinal;
   Reserved, dwFlags: Cardinal;
begin
   if (Assigned(Binding) and (Binding.QueryInterface(IWinInetHttpInfo, HttpInfo) = S_OK)) then
      begin
         Info := 0;
         Reserved := 0;
         dwFlags := 0;
         BufferLength := SizeOf(Cardinal);
         Result := not Boolean(HttpInfo.QueryInfo(dwOption or HTTP_QUERY_FLAG_NUMBER,
            @C, BufferLength, dwflags, reserved));
         HttpInfo := nil;
         if Result then
            Info := C;
      end
   else
      Result := False;
end;

function TBSCB.QueryInfo(dwOption: DWORD; var info: string): Boolean;
var

   buf: array[0..INTERNET_MAX_PATH_LENGTH] of char;
   HttpInfo: IWinInetHttpInfo;
   buflength, dwReserved, dwFlags: Cardinal;
begin
   if (Assigned(Binding) and (Binding.QueryInterface(IWinInetHttpInfo, HttpInfo) = S_OK)) then
      begin
         Info := '';
         dwReserved := 0;
         dwFlags := 0;
         buflength := INTERNET_MAX_PATH_LENGTH + 1;
         Result := not Boolean(HttpInfo.QueryInfo(dwOption, @buf, bufLength, dwFlags, dwReserved));
         HttpInfo := nil;
         if Result then
            Info := Buf;
      end
   else
      Result := False;
end;

function TBSCB.QueryInfo(dwOption: DWORD; var Info: TDateTime): Boolean;
var
   HttpInfo: IWinInetHttpInfo;
   SysTime: TSystemtime;
   BufferLength: Cardinal;
   Reserved, dwFlags: Cardinal;
begin
   if (Assigned(Binding) and (Binding.QueryInterface(IWinInetHttpInfo, HttpInfo) = S_OK)) then
      begin
         Info := 0;
         Reserved := 0;
         dwFlags := 0;
         BufferLength := SizeOf(TSystemTime);
         Result := not Boolean(HttpInfo.QueryInfo(dwOption or HTTP_QUERY_FLAG_SYSTEMTIME,
            @SysTime, BufferLength, dwflags, Reserved));
         HttpInfo := nil;
         if Result then
            Info := SystemTimeToDateTime(SysTime);
      end
   else
      Result := False;
end;

function TBSCB.GetBindResult(out clsidProtocol: TCLSID; out dwResult: DWORD;
   out szResult: POLEStr): HResult;
var
   dwReserved: DWORD;
begin
   dwReserved := 0;
   Result := Binding.GetBindResult(clsidProtocol, dwResult, szResult, dwReserved);
end;

function TBSCB.GetSaveFileAs: WideString;
begin
  Result := Info.DownloadDir + Info.FileName;
 {	if (Copy(Info.DownloadDir, Length(Info.DownloadDir), 1) = '\') then
  Result := Info.DownloadDir + Info.FileName
  else
    Result := Info.DownloadDir + '\' + Info.FileName; }
end;

function GetDownloadDirectory: WideString;
var
   Reg: TRegistry;
begin
   Result := '';
   Reg := TRegistry.Create;
   try
      Reg.RootKey := HKEY_CURRENT_USER;
      if Reg.OpenKeyReadOnly('Software\Microsoft\Internet Explorer') then
         begin
            Result := Reg.ReadString('Download Directory');
         end;
   finally
      Reg.Free;
   end;
end;

constructor TIEDownload.Create(AOwner: TComponent);
begin
   inherited;
   FCodepageOption := Ansi;
   FProxySettings := TProxySettings.Create;
   FProxySettings.FPort := 80;
   FUrlEncodeOptions := [];
   FDefaultProtocol := 'http://';
   FObjList := TObjectList.Create;
   FAdditionalHeader := TStringlist.Create;
   FAdditionalHeader.Add('Content-Type: application/x-www-form-urlencoded ');
   FRange := TRange.Create;
   FSecurity := TSecurity.Create;
   FRefCount := 1;
   FBindInfoOptions := [Asynchronous, AsyncStorage, PullData, NoWriteCache, GetNewestVersion];
   FBindVerbOption := Get; //Method;
   FCodePageOption := Ansi; //Ansi Code page;
   UpdateUserAgent;
end;

procedure TIEDownload.Loaded;
begin
   inherited Loaded;
   UpdateUserAgent;
   Busy := False;
   FBusy := Busy;
   if (FProxySettings.FAutoLoadProxy) and (FProxySettings.FServer <> '') then
      FProxySettings.SetProxy(FullUserAgent, FProxySettings.FServer + ':' + IntToStr(FProxySettings.FPort));
end;

destructor TIEDownload.Destroy;
begin
   Busy := False;
   FBusy := Busy;
   FObjlist.Free;
   FRange.Free;
   FSecurity.Free;
   FAdditionalHeader.Free;
   if FProxySettings.FAutoLoadProxy then
      FProxySettings.SetProxy('', '');
   FProxySettings.Free;
   inherited;
end;

procedure TIEDownload.UpdateUserAgent;
begin
   FullUserAgent := 'User-agent: Mozilla/4.0 (Compatible ' + FUserAgent + ' by IEDownload- from: ' + SITE_ADDRESS + ')' + #13#10
end;

function TIEDownload.SetDownloadDir: Boolean;
begin
   if FDownloadDir = '' then
      FDownloadDir := ExtractFilePath(Application.ExeName) + 'Downloads\';
   Result := CreateDir(FDownloadDir);
end;

procedure TIEDownload.Cancel;
begin
   FCancel := True;
end;

procedure TIEDownload.Cancel(Item: TBSCB);
begin
   Item.FState := Canceled;
end;

procedure TIEDownload.FillInfo(var Info: TInfo);
begin
   with Info do
      begin
         Sender := Self;
         DownloadDir := Self.DownloadDir;
         AdditionalHeader.AddStrings(FAdditionalHeader);
         BindVerbValue := FBindVerbValue;
         BindInfoValue := FBindInfoValue;
         Password := FPassword;
         UserName := FUserName;
         CustomVerb := FCustomVerb;
         ExtraInfo := FExtraInfo;
         PostData := FPostData;
         PutFileName := FPutFileName;
         Descriptor := Security.Descriptor;
         InheritHandle := Security.InheritHandle;
         RangeBegin := Range.RangeBegin;
         RangeEnd := Range.RangeEnd;
         UrlEncodeValue := FUrlEncodeValue;
         CodePageValue := FCodepageValue;
         TimeOut := FTimeOut;
         OpenAfterDownload := False;
         FileSize := 0;
         UserAgent := FullUserAgent; //Is it needed here?
      end;
end;

procedure TIEDownload.Go(URL: WideString);
var
   Info: TInfo;
   BS: TBSCB;
begin
   FCancel := False;
   FURL := URL;
   FURL := AddPrefixHttp;
   Info.Url := StringToOleStr(FURL);
   Info.FileName := '';
   Info.AdditionalHeader := TStringlist.Create;
   FillInfo(Info);
   BS := TBSCB.Create(Info);
   if FBindInfoValue <> FBindInfoValue or BINDF_ASYNCHRONOUS then
      BS.Free
   else
      FObjList.Add(BS);
end;

procedure TIEDownload.Download(pmk: IMoniker; pbc: IBindCtx);
var
   Info: TInfo;
   BS: TBSCB;
begin
   FCancel := False;
   Info.Url := '';
   Info.FileName := '';
   Info.AdditionalHeader := TStringlist.Create;
   FillInfo(Info);
   BS := TBSCB.Create(Info, pmk, pbc);
   if FBindInfoValue <> FBindInfoValue or BINDF_ASYNCHRONOUS then
      BS.Free
   else
      FObjList.add(BS);
end;

procedure TIEdownload.UpdateUrlEncodeValue;
const
   acardUrlEncodeValues: array[TUrlEncodeOption] of Cardinal = (
      $00000001, $00000002);
var
   i: TUrlEncodeOption;
begin
   FUrlEncodeValue := 0;
   if (FUrlEncodeOptions <> []) then
      for i := Low(TUrlEncodeOption) to High(TUrlEncodeOption) do
         if (i in FUrlEncodeOptions) then
            Inc(FUrlencodeValue, acardUrlEncodeValues[i]);
end;

procedure TIEDownload.SetUrlEncodeOptions(const Value: TUrlEncodeOptions);
begin
   FUrlEncodeOptions := Value;
   UpdateUrlEncodeValue;
end;

procedure TIEDownload.SetDownloadMethod(const Value: TDownloadMethod);
begin
   FDownloadMethod := Value;
end;

procedure TIEDownload.UpdateBindInfoValue;
const
   acardBindInfoValues: array[TBindInfoOption] of Cardinal = (
      $00000001, $00000002, $00000004, $00000008, $00000010, $00000020, $00000040, $00000080,
      $00000100, $00000200, $00000400, $00000800, $00001000, $00002000, $00004000, $00008000,
      $00010000, $00020000, $00040000, $00080000, $00100000, $00200000, $00400000, $00800000);
var
   i: TBindInfoOption;
begin
   FBindInfoValue := 0;
   if (FBindInfoOptions <> []) then
      for i := Low(TBindInfoOption) to High(TBIndInfoOption) do
         if (i in FBindInfoOptions) then
            Inc(FBindInfoValue, acardBindInfoValues[i]);
end;

procedure TIEDownload.SetBindInfoOptions(const Value: TBindInfoOptions);
begin
   FBindInfoOptions := Value;
   UpdateBindInfoValue;
end;

procedure TIEDownload.SetCodePageOption(const Value: TCodepageOption);
begin
   FCodepageOption := Value;
   case FCodepageOption of
      Ansi: FCodepageValue := CP_ACP;
      Mac: FCodepageValue := CP_MACCP;
      OEM: FCodepageValue := CP_OEMCP;
      Symbol: FCodepageValue := CP_SYMBOL;
      ThreadsAnsi: FCodepageValue := CP_THREAD_ACP;
      UTF7: FCodepageValue := CP_UTF7;
      UTF8: FCodepageValue := CP_UTF8;
   end;
end;

procedure TIEDownload.SetBindVerbOption(const Value: TBindVerbOption);
begin
   FBindVerbOption := Value;
   case FBindVerbOption of
      Get: FBindVerbValue := BINDVERB_GET;
      Put: FBindVerbValue := BINDVERB_PUT;
      Post: FBindVerbValue := BINDVERB_POST;
      Custom: FBindVerbValue := BINDVERB_CUSTOM;
   end;
end;

procedure TIEDownload.SetAdditionalHeader(const Value: Tstrings);
begin
   FAdditionalHeader.Assign(Value);
end;

function TIEDownload.AddPrefixHttp: Widestring;
var
   st: string;
begin
   st := AnsiUpperCase(FURL);
   if (AnsiPos('HTTP', st) <> 0)
      then
      Result := FURL
   else
      Result := FDefaultProtocol + FURL;
end;

procedure TIEDownload.SetDefaultProtocol(const Value: string);
begin
   FDefaultProtocol := (Value);
   if fDefaultProtocol = '' then
      FDefaultProtocol := 'http://';
end;

procedure TIEDownload.BusyChange;
begin
   if Assigned(FOnBusy) then
      FOnBusy(Self);
end;

procedure TBSCB.UpdateEvents(Code: integer; BusyState: Boolean; Sender: TBSCB;
   ulProgress, ulProgressMax: ULONG; szStatusText: LPCWSTR; Downloaded, ElapsedTime,
   Speed, RemainingTime, Status: string; Stream: TStream);
begin
   with FSender do
      begin
         FBusy := BusyState;
         Busy := FBusy;
         BusyChange;
         if (Code >= 0) and (Code < 99) then
            if Assigned(FOnStatusText) then
               FOnStatusText(Self, BindStatusText(Code));
         if (Code = 0) or ((Code > 99) and (Code < 506)) then
            if Assigned(FOnRespondText) then
               FOnRespondText(Self, ResponseCodeText(Code));
         if (Code < -10000) or (Code > 12000) then
            if Assigned(FOnErrorText) then
               begin
                  FOnErrorText(Self, ErrorText(Code));
                  if Assigned(FOnRespondText) then
                     FOnRespondText(Self, '');
                  if Assigned(FOnStatusText) then
                     FOnStatusText(Self, 'Download Error.');
               end;
 //           if Assigned(FOnComplete) then
 //               FOnComplete(Self, Stream, Code);
         if Assigned(FOnProgress) then
            OnProgress(Sender, ulProgress, ulProgressMax, Code, szStatusText,
               Downloaded, ElapsedTime, Speed, RemainingTime, Status);
      end;
end;

//--TProxy Section--------------------------------------------------------------
type
   INTERNET_PER_CONN_OPTION = record
      dwOption: DWORD;
      Value: record
         case Integer of
            1: (dwValue: DWORD);
            2: (pszValue: PAnsiChar);
            3: (ftValue: TFileTime);
      end;
   end;
   LPINTERNET_PER_CONN_OPTION = ^INTERNET_PER_CONN_OPTION;
   INTERNET_PER_CONN_OPTION_LIST = record
      dwSize: DWORD;
      pszConnection: LPTSTR;
      dwOptionCount: DWORD;
      dwOptionError: DWORD;
      pOptions: LPINTERNET_PER_CONN_OPTION;
   end;
   LPINTERNET_PER_CONN_OPTION_LIST = ^INTERNET_PER_CONN_OPTION_LIST;

procedure TProxySettings.SetProxy(FullUserAgent, ProxyServer: string); //mladen
var
   list: INTERNET_PER_CONN_OPTION_LIST;
   dwBufSize: DWORD;
   hInternet: Pointer;
   Options: array[1..3] of INTERNET_PER_CONN_OPTION;
const
   INTERNET_PER_CONN_FLAGS = 1;
   INTERNET_PER_CONN_PROXY_SERVER = 2;
   INTERNET_PER_CONN_PROXY_BYPASS = 3;
   INTERNET_PER_CONN_AUTOCONFIG_URL = 4;
   INTERNET_PER_CONN_AUTODISCOVERY_FLAGS = 5;
   PROXY_TYPE_DIRECT = $00000001; // direct to net
   PROXY_TYPE_PROXY = $00000002; // via named proxy
   PROXY_TYPE_AUTO_PROXY_URL = $00000004; // autoproxy URL
   PROXY_TYPE_AUTO_DETECT = $00000008; // use autoproxy detection
   INTERNET_OPTION_REFRESH = 37;
   INTERNET_OPTION_PER_CONNECTION_OPTION = 75;
   INTERNET_OPTION_SETTINGS_CHANGED = 39;
begin
   dwBufSize := sizeof(list);
   list.dwSize := sizeof(list);
   list.pszConnection := nil;
   list.dwOptionCount := High(Options); // the highest index of the array (in this case 3)
   Options[1].dwOption := INTERNET_PER_CONN_FLAGS;
   Options[1].Value.dwValue := PROXY_TYPE_DIRECT or PROXY_TYPE_PROXY;
   Options[2].dwOption := INTERNET_PER_CONN_PROXY_SERVER;
   Options[2].Value.pszValue := PChar(ProxyServer);
   Options[3].dwOption := INTERNET_PER_CONN_PROXY_BYPASS;
   Options[3].Value.pszValue := '<Download>'; // '<Download>' instead of 'Download'
   list.pOptions := @Options;
   hInternet := InternetOpen(PChar(FullUserAgent), INTERNET_OPEN_TYPE_DIRECT, nil, nil, 0);
   InternetSetOption(hInternet, INTERNET_OPTION_PER_CONNECTION_OPTION, @list, dwBufSize);
   InternetSetOption(hInternet, INTERNET_OPTION_REFRESH, nil, 0);
   InternetCloseHandle(hInternet);
end;
// End of Proxy-----------------------------------------------------------------

initialization
   coInitialize(nil);
finalization
   coUninitialize;
end.

