{ ***********************************************************
//                     TUILess - UI_Less parser             *
//                       For Delphi                         *
//                                                          *
//                     Freeware Component                   *
//                            by                            *
//                     Per Linds Larsen                    *                                                       //                   per.lindsoe@larsen.dk                  *
//                     Developing Team:                     *
//                  bsalsa - bsalsa.com                     *
//  Documentation and updated versions:                     *
//                                                          *
//               http://www.bsalsa.com                      *
{*******************************************************************************}
{LICENSE:
THIS SOFTWARE IS PROVIDED TO YOU "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESSED OR IMPLIED INCLUDING BUT NOT LIMITED TO THE APPLIED
WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
YOU ASSUME THE ENTIRE RISK AS TO THE ACCURACY AND THE USE OF THE SOFTWARE
AND ALL OTHER RISK ARISING OUT OF THE USE OR PERFORMANCE OF THIS SOFTWARE
AND DOCUMENTATION. [YOUR NAME] DOES NOT WARRANT THAT THE SOFTWARE IS ERROR-FREE
OR WILL OPERATE WITHOUT INTERRUPTION. THE SOFTWARE IS NOT DESIGNED, INTENDED
OR LICENSED FOR USE IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE CONTROLS,
INCLUDING WITHOUT LIMITATION, THE DESIGN, CONSTRUCTION, MAINTENANCE OR
OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS,
AIR TRAFFIC CONTROL, AND LIFE SUPPORT OR WEAPONS SYSTEMS. VSOFT SPECIFICALLY
DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR SUCH PURPOSE.

You may use, change or modify the component under 3 conditions:
1. In your website, add a link to "http://www.bsalsa.com"
2. In your application, add credits to "Embedded Web Browser"
3. Mail me  (bsalsa@bsalsa.com) any code change in the unit
   for the benefit of the other users.
4. Please, consider donation in our web site!
{*******************************************************************************}

unit UI_Less;

interface

uses
   MsHtml_EWB, Urlmon, ActiveX, Windows, Messages, Classes;

const
   WM_USER_STARTWALKING = WM_USER + 1;
   DISPID_AMBIENT_DLCONTROL = (-5512);
   READYSTATE_COMPLETE = $00000004;

type
   TInvokeEvent = function(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params: TagDispParams; VarResult, ExcepInfo, ArgErr: Pointer): HRESULT of object;

   TUILess = class(TComponent, IUnknown, IDispatch, IPropertyNotifySink, IOleClientSite)
   private
      FOnInvoke: TInvokeEvent;
      FAbout: string;
      FEnabled: Boolean;

      procedure SetAbout(Value: string);
   protected
/// IDISPATCH
      function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
         Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
/// IPROPERTYNOTIFYSINK
      function OnChanged(dispid: TDispID): HResult; stdcall;
      function OnRequestEdit(dispid: TDispID): HResult; stdcall;
/// IOLECLIENTSITE
      function SaveObject: HResult; stdcall;
      function GetMoniker(dwAssign: Longint; dwWhichMoniker: Longint;
         out mk: IMoniker): HResult; stdcall;
      function GetContainer(out container: IOleContainer): HResult; stdcall;
      function ShowObject: HResult; stdcall;
      function OnShowWindow(fShow: BOOL): HResult; stdcall;
      function RequestNewObjectLayout: HResult; stdcall;
///
      function LoadUrlFromMoniker: HResult;
      function LoadUrlFromFile: HResult;
// * We only use LoadUrlFromMoniker, but we could use LoadUrlFromFile instead.

   public
      HtmlElementCollection: IHtmlElementCollection;
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      function Go(URL: WideString): IHTMLELEMENTCollection;
      procedure GetImageList(ImagesList: TStrings);
      procedure GetAnchorList(Anchorlist: TStrings); overload;
      procedure GetLinkList(LinkList: TStrings); overload;
   published
      property OnInvoke: TInvokeEvent read FOnInvoke write FOnInvoke;

      property Enabled: Boolean read FEnabled write FEnabled default True;
      property About: string read fAbout write SetAbout;
   end;

implementation

var
   Doc: IhtmlDocument2;
   _URL: WideString;

/// CORE ---->>>>>>>>>

constructor TUILess.Create;
begin
   FAbout := 'TUILess parser  - Help & Support: http://www.bsalsa.com/';
   FEnabled := True;
   inherited;
end;

destructor TUILess.Destroy;
begin
   inherited;
end;

function TUILess.Go(URL: WideString): IHtmlElementCollection;
var
   Cookie: Integer;
   CP: IConnectionPoint;
   OleObject: IOleObject;
   OleControl: IOleControl;
   CPC: IConnectionPointContainer;
   Msg: TMsg;
   hr: HRESULT;
begin
   if FEnabled then
      begin
         _URL := Url;
         CoCreateInstance(CLASS_HTMLDocument, nil, CLSCTX_INPROC_SERVER, IID_IHTMLDocument2, Doc);
         OleObject := Doc as IOleObject;
         OleObject.SetClientSite(self);
         OleControl := Doc as IOleControl;
         OleControl.OnAmbientPropertyChange(DISPID_AMBIENT_DLCONTROL);
         CPC := Doc as IConnectionPointContainer;
         CPC.FindConnectionPoint(IpropertyNotifySink, CP);
         CP.Advise(self, Cookie);
         HR := LoadUrlFromMoniker; // alternative: Hr:= LoadUrlFromFile;

         if ((SUCCEEDED(HR)) or (HR = E_PENDING)) then
            while (GetMessage(msg, 0, 0, 0)) do
               if ((msg.message = WM_USER_STARTWALKING) and (msg.hwnd = 0)) then
                  begin
                     PostQuitMessage(0);
                     HtmlElementCollection := Doc.Get_all;
                     Result := HtmlElementCollection;
                  end
               else
                  DispatchMessage(msg);
      end;
end;

function TUILess.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
   Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult;
const
   DLCTL_NO_SCRIPTS = $00000080;
   DLCTL_NO_JAVA = $00000100;
   DLCTL_NO_RUNACTIVEXCTLS = $00000200;
   DLCTL_NO_DLACTIVEXCTLS = $00000400;
   DLCTL_DOWNLOADONLY = $00000800;

var
   I: Integer;
begin
   Result := E_Fail;
   if FEnabled then
      begin
         if DISPID_AMBIENT_DLCONTROL = DispID then
            begin
               I := DLCTL_DOWNLOADONLY + DLCTL_NO_SCRIPTS +
                  DLCTL_NO_JAVA + DLCTL_NO_DLACTIVEXCTLS +
                  DLCTL_NO_RUNACTIVEXCTLS;
               PVariant(VarResult)^ := I;
               Result := S_OK;
               if Assigned(FOnInvoke) then
                  FOnInvoke(DispID, IID, LocaleID, Flags, TagDispParams(Params),
                     VarResult, ExcepInfo, ArgErr)
            end
         else
            Result := DISP_E_MEMBERNOTFOUND;
      end;
end;

function TUILess.OnChanged(dispid: TDispID): HResult;
var
   dp: TDispParams;
   vResult: OleVariant;
begin
   Result := E_Fail;
   if FEnabled then
      begin
         if (DISPID_READYSTATE = Dispid) then
            begin
               if SUCCEEDED((Doc as Ihtmldocument2).Invoke(DISPID_READYSTATE, GUID_null,
                  LOCALE_SYSTEM_DEFAULT, DISPATCH_PROPERTYGET, dp, @vresult, nil, nil)) then
                  if Integer(vResult) = READYSTATE_COMPLETE then
                     PostThreadMessage(GetCurrentThreadId(), WM_USER_STARTWALKING, 0, 0);
            end;
      end;
end;

function TUILess.LoadUrlFromMoniker: HResult;
var
   Moniker: IMoniker;
   BindCtx: IBindCTX;
   PM: IPersistMoniker;
begin
   Result := E_Fail;
   if FEnabled then
      begin
         createURLMoniker(nil, PWideChar(_URL), Moniker);
         CreateBindCtx(0, BindCtx);
         PM := Doc as IPersistMoniker;
         Result := PM.Load(LongBool(0), Moniker, BindCtx, STGM_READ);
      end;
end;

function TUILess.LoadUrlFromFile: HResult;
var
   PF: IPersistfile;
begin
   Result := E_Fail;
   if FEnabled then
      begin
         PF := Doc as IPersistfile;
         Result := PF.Load(PWideChar(_URL), 0);
      end;
end;

///  UTILILIES ---------- >>>>>>>>>>>>>>>>>>>>>

procedure TUILess.GetImageList(ImagesList: TStrings);
var
   ImageElement: IHTMLImgElement;
   Disp: IDispatch;
   x: Integer;
begin
   if HtmlElementCollection <> nil then
      begin
         for x := 0 to HtmlElementCollection.length - 1 do
            begin
               Disp := HtmlElementCollection.item(x, 0);
               if SUCCEEDED(Disp.QueryInterface(IHTMLImgElement, ImageElement))
                  then
                  ImagesList.add(ImageElement.src);
            end;
      end;
end;

procedure TUILess.GetAnchorList(Anchorlist: TStrings);
var
   Anchor: IHTMLAnchorElement;
   Disp: IDispatch;
   x: Integer;
begin
   if HtmlElementCollection <> nil then
      begin
         for x := 0 to HtmlElementCollection.length - 1 do
            begin
               Disp := HtmlElementCollection.item(x, 0);
               if SUCCEEDED(Disp.QueryInterface(IHTMLAnchorElement, Anchor))
                  and (anchor.href <> '')
                  then
                  Anchorlist.Add(Anchor.href);
            end;
      end;
end;

procedure TUILess.GetLinkList(LinkList: TStrings);
var
   Anchor: IHTMLAnchorElement;
   Link: IHTMLElement;
   Disp: IDispatch;
   x: Integer;
begin
   if HtmlElementCollection <> nil then
      begin
         for x := 0 to HtmlElementCollection.length - 1 do
            begin
               Disp := HtmlElementCollection.item(x, 0);
               if SUCCEEDED(Disp.QueryInterface(IHTMLElement, Link)) and
                  (Link.innerText <> '') then
                  begin
                     if SUCCEEDED(Disp.QueryInterface(IHTMLAnchorElement, Anchor))
                        and (Anchor.href <> '') then
                        LinkList.Add(Link.innerText);
                  end;
            end;
      end;
end;

/// Don't Care ------>>>>>>>>>>>

procedure TUILess.SetAbout(Value: string);
begin
   Exit;
end;

function TUILess.OnRequestEdit(dispid: TDispID): HResult;
begin
   RESULT := E_NOTIMPL;
end;

function TUILess.SaveObject: HResult;
begin
   result := E_NOTIMPL;
end;

function TUILess.GetMoniker(dwAssign: Longint; dwWhichMoniker: Longint;
   out mk: IMoniker): HResult;
begin
   result := E_NOTIMPL;
end;

function TUILess.GetContainer(out container: IOleContainer): HResult;
begin
   result := E_NOTIMPL;
end;

function TUILess.ShowObject: HResult;
begin
   result := E_NOTIMPL;
end;

function TUILess.OnShowWindow(fShow: BOOL): HResult;
begin
   result := E_NOTIMPL;
end;

function TUILess.RequestNewObjectLayout: HResult;
begin
   result := E_NOTIMPL;
end;

end.

